<?php


namespace Controller;

use Service\Issue as IssueService;
use Traits\TResponse;
use Util\InstanceProvider;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\DevOps\Util\Utilities as DevopsUtilities;

class Issue
{
    use TResponse;

    /**
     * @var IssueService
     */
    private $issueService;

    public function __construct(IssueService $issueService)
    {
        $this->issueService = $issueService;
    }

    public function findActionsOfExits(Request $request, Response $response)
    {
        try {
            $issueId = $request->getParameter('issueId');
            $actions = $this->issueService->getActionsOfExits($issueId);
            $this->sendResponse($actions, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function saveSendOutComment(Request $request, Response $response)
    {
        $params = $request->getParameters();
        try {
            $description = $params['DESCRIPTION'];
            if (!empty($params['DOCUMENT'])) {
                InstanceProvider::getFileUploaderServe()->uploadFilesInRow($params, 'DOCUMENT', 'PmDevOps/Task');
                $document = $params['DOCUMENT'];
            } else {
                $document = array();
            }

            $taskTypeTarget = $params['TASK_TYPE_TARGET'];
            $commentTypeId = $params['COMMENT_TYPE_ID'];
            $issueId = $params['ISSUE_ID'];
            $specialFunction = empty($params['SPECIAL_FUNCTION']) ? null : $params['SPECIAL_FUNCTION'];

            $this->issueService->sendOutComment($issueId, $document, $taskTypeTarget, $commentTypeId, $description, $specialFunction);
        } catch (\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }
    public function clientRefuse(Request $request, Response $response)
    {
        $params = $request->getParameters();
        try {
            $description = $params['DESCRIPTION'];
            if (!empty($params['DOCUMENT'])) {
                InstanceProvider::getFileUploaderServe()->uploadFilesInRow($params, 'DOCUMENT', 'PmDevOps/Task');
                $document = $params['DOCUMENT'];
            } else {
                $document = array();
            }

            $commentTypeId = $params['COMMENT_TYPE_ID'];
            $issueId = $params['ISSUE_ID'];

            $this->issueService->clientRefuse($issueId, $commentTypeId, $description, $document);
        } catch (\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }
    public function closeIssue(Request $request, Response $response)
    {
        $params = $request->getParameters();
        try {
            $issueId = $params['ISSUE_ID'];

            $this->issueService->closeIssue($issueId);
        } catch (\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function issueBudgetSave(Request $request, Response $response)
    {
        $params = $request->getParameters();
        try {
            $issueId = $params['CALL_RELATIONSHIP'];
            $callType = $params['CALL_TYPE'];
            $data = $params['DATA'];
            $dataMaintenance = $params['DATA_MAINTENANCE'];

            $this->issueService->issueBudgetSave($data);
            $this->issueService->saveMaintenanceCost($dataMaintenance);

            $this->sendResponse([], 200);
        } catch (\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function issueBudgetInstallmentsSave(Request $request, Response $response)
    {
        $params = $request->getParameters();
        try {
            $issueId = $params['CALL_RELATIONSHIP'];
            $data = $params['DATA'];

            $this->issueService->issueBudgetPlotsSave($data);

            $this->sendResponse([], 200);
        } catch (\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function maintenanceCost(Request $request, Response $response)
    {
        try {
            $params = $request->getParameters();
            $issueId = $params['CALL_RELATIONSHIP'];

            $budget = $this->issueService->findMaintenanceCost($issueId);
            $data = array(
                "data" => $budget,
                "pagination" => array(
                    "limit" => 500000,
                    "page" => 1,
                    "total" => count($budget)
                )
            );
            $this->sendResponse($data, 200);
        } catch (\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function issueBudget(Request $request, Response $response)
    {
        try {
            $conditions = $request->getParameters();
            $budget = [];
            $issueId = empty($conditions['ISSUE']) ? null : $conditions['ISSUE'];
            $nameProposal = empty($conditions['NAME_PROPOSAL']) ? null : $conditions['NAME_PROPOSAL'];
            $namePercent = empty($conditions['NAME_PERCENT']) ? null : $conditions['NAME_PERCENT'];
            $nameDiscount = empty($conditions['NAME_DISCOUNT']) ? null : $conditions['NAME_DISCOUNT'];
            if ($issueId) {
                $budget = InstanceProvider::getService()->getIssue()->findIssueBudget(
                    $issueId,
                    $nameProposal,
                    $nameDiscount,
                    $namePercent
                );
            }
            foreach ($budget as &$item) {
                $item['PROPOSAL_VALUE'] = intval($item['PROPOSAL_VALUE']);
            }
            $data = array(
                "data" => $budget,
                "pagination" => array(
                    "limit" => 500000,
                    "page" => 1,
                    "total" => count($budget)
                )
            );
            $this->sendResponse($data, 200);
        } catch (\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function issueBudgetPlots(Request $request, Response $response)
    {
        try {
            $conditions = $request->getParameters();
            $budget = [];
            $issueId = empty($conditions['ISSUE']) ? null : $conditions['ISSUE'];
            $namePlots = empty($conditions['NAME_PLOTS']) ? null : $conditions['NAME_PLOTS'];
            if ($issueId) {
                $budget = InstanceProvider::getService()->getIssue()->findIssueBudgetPlots($issueId, $namePlots);
            }
            foreach ($budget as &$item) {
                $item['PROPOSAL_VALUE'] = intval($item['PROPOSAL_VALUE']);
                $item['VALUE_PLOTS'] = intval($item['VALUE_PLOTS']);
                $item['DAYS_PAYMENT'] = intval($item['DAYS_PAYMENT']);
                $item['ORDER_PLOTS_NUM'] = intval($item['ORDER_PLOTS_NUM']);
            }
            $data = array(
                "data" => $budget,
                "pagination" => array(
                    "limit" => 500000,
                    "page" => 1,
                    "total" => count($budget)
                )
            );
            $this->sendResponse($data, 200);
        } catch (\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function findEntranceCommentIssueStatus(Request $request, Response $response)
    {
        try {
            $filter = $request->getParameters();
            $issueStatus = $filter['ISSUE_STATUS'];
            $issues = $this->issueService->findEntranceCommentIssueStatus($issueStatus);
            $this->sendResponse($issues, 200);
        } catch (\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function findIssueFramingClassification(Request $request, Response $response)
    {
        try {
            $filter = $request->getParameters();
            $issueId = null;
            if (!empty($filter['ISSUE'])) {
                $issueId = $filter['ISSUE'];
                unset($filter['ISSUE']);
            }
            $issues = $this->issueService->findIssueFramingClassification($issueId);
            $this->sendResponse($issues, 200);
        } catch (\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function findIssueFramingProjectType(Request $request, Response $response)
    {
        try {
            $filter = $request->getParameters();
            $issueId = null;
            if (!empty($filter['ISSUE'])) {
                $issueId = $filter['ISSUE'];
                unset($filter['ISSUE']);
            }
            $issues = $this->issueService->findIssueFramingProjectType($issueId);
            $this->sendResponse($issues, 200);
        } catch (\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function findOutputCommentIssueStatus(Request $request, Response $response)
    {
        try {
            $filter = $request->getParameters();
            $issueStatus = null;
            if (!empty($filter['ISSUE_STATUS'])) {
                $issueStatus = $filter['ISSUE_STATUS'];
                unset($filter['ISSUE_STATUS']);
            }
            $issues = $this->issueService->findOutputCommentIssueStatus($issueStatus);
            $this->sendResponse($issues, 200);
        } catch (\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function removeIssueFramingClassification(Request $request, Response $response)
    {
        $params = $request->getParameters()['data'];
        try {
            foreach ($params as $param) {
                $taskTypeClassificationId = $param['ID'];
                $this->issueService->removeTaskTypeClassification($taskTypeClassificationId);
            }
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function removeIssueFramingProjectType(Request $request, Response $response){
        $params = $request->getParameters()['data'];
        try {
            foreach ($params as $param) {
                $id = $param['ID'];
                $this->issueService->removeIssueFramingProjectType($id);
            }
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function saveAddTaskClassification(Request $request, Response $response){
        $issueClassifications = $request->getParameters();
        try {
            $taskTypeId = $issueClassifications['TASK_TYPE'];
            $this->issueService->saveAddTaskClassification($issueClassifications, $taskTypeId);
        }
        catch(\Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function findCommentTypeIssueStatus(Request $request, Response $response){
        try {
            $issues = $this->issueService->findCommentTypeIssueStatus();
            $this->sendResponse($issues, 200);
        } catch(\Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function saveOutputCommentIssueStatus(Request $request, Response $response){
        $params = $request->getParameters();
        try {
            $action = $params['ACTION_LABEL'];
            $issueClassification = $params['ISSUE_CLASSIFICATION'];
            $description = $params['DESCRIPTION'];
            $commentType = $params['COMMENT_TYPE'];
            $targetStatus = $params['TARGET_STATUS'];
            $taskType = $params['TASK_TYPE'];
            $outputCommentId = empty($params['ID']) ? 0 : $params['ID'];
            $this->issueService->saveOutputCommentIssueStatus($outputCommentId, $action, $issueClassification, $description, 
            $commentType, $targetStatus, $taskType);
            
        }
        catch(\Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function removeOutputCommentIssueStatus(Request $request, Response $response){
        $params = $request->getParameters()['data'];
        try {
            foreach ($params as $param) {
                $id = $param['ID'];
                $this->issueService->removeOutputCommentIssueStatus($id);
            }
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function saveAddProjectType(Request $request, Response $response){
        $projectTypes = $request->getParameters();
        try {
            $taskTypeId = $projectTypes['TASK_TYPE'];
            $this->issueService->saveAddProjectType($projectTypes, $taskTypeId);
        }
        catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function saveIssueOrderToFrame(Request $request, Response $response) {
        $issue = $request->getParameters();
        try {
            $orderToFrame = $issue['ORDER_TO_FRAME'];
            $taskTypeId = $issue['ID'];
            $allowPlayTaskInFrame = $issue['ALLOW_PLAY_TASK_IN_FRAME'];
            $this->issueService->saveIssueOrderToFrame($taskTypeId,$orderToFrame,$allowPlayTaskInFrame);
        }
        catch(\Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function saveEntranceCommentIssueStatus(Request $request, Response $response){
        $rows = $request->getParameters();
        try {
            $commentType = $rows['COMMENT_TYPE'];
            $description = $rows['DESCRIPTION'];
            $issueClassificationId = $rows['ISSUE_CLASSIFICATION'];
            $taskTypeId = $rows['TASK_TYPE'];
            $this->issueService->saveEntranceCommentIssueStatus($taskTypeId,$issueClassificationId,$description,$commentType);
        }
        catch(\Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
}
