<?php

namespace Repository;

use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Util\InstanceProvider;
use Zeedhi\DevOps\Util\Utilities;

class TaskTypeClassification extends \Doctrine\ORM\EntityRepository
{

    public function findTasksWithFraming($issueId)
    {
        $query = <<<SQL
       SELECT TAS.ID
         FROM TASK ISS
         JOIN TASK_RELATIONSHIP TAR ON (TAR.TASK = ISS.ID AND TAR.TYPE = :typeIssue)
         JOIN TASK TAS ON (TAS.ID = TAR.TASK_RELATIONSHIP)
         JOIN TASK_STATUS TST ON (TST.ID = ISS.TASK_STATUS)
         JOIN TASK_TYPE_CLASSIFICATION TTC ON (TTC.TASK_TYPE_ID = TST.TASK_TYPE AND TTC.TASK_ISSUE_CLASSIF = :typeLocked)
        WHERE ISS.ID = :issueId
          AND ISS.STATUS = :statusActive
          AND TAS.STATUS = :statusActive
          AND TAS.PERCENT_COMPLETED < 100
          AND TAS.CLASSIFICATION = TTC.CLASSIFICATION_ID
SQL;
        $params = array(
            'issueId' => $issueId,
            'typeLocked' => Entities\TaskTypeClassification::TYPE_LOCKED,
            'typeIssue' => Entities\TaskRelationship::RELATIONSHIP_ISSUE_TYPE,
            'statusActive' => Utilities::STATUS_ACTIVE,
        );

        $return = $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query, $params);

        return $return;
    }

    public function findTaskTypesForFraming($issueId)
    {
        $query = <<<SQL
       SELECT TTY.INITIAL_TASK_STATUS,TTY.ID,TTY.ORDER_TO_FRAME
         FROM TASK ISS
         JOIN TASK_RELATIONSHIP TAR ON (TAR.TASK = ISS.ID AND TAR.TYPE = :typeIssue)
         JOIN TASK TAS ON (TAS.ID = TAR.TASK_RELATIONSHIP AND TAS.USED_IN_FRAMING = :usedInFraming)
         JOIN TASK_STATUS TST ON (TST.ID = ISS.TASK_STATUS)
         JOIN TASK_TYPE_CLASSIFICATION TTC ON ( TAS.CLASSIFICATION = TTC.CLASSIFICATION_ID AND TTC.TASK_ISSUE_CLASSIF = :typeLocked)
         JOIN TASK_TYPE TTY ON (TTC.TASK_TYPE_ID = TTY.ID)
        WHERE ISS.ID = :issueId
          AND ISS.STATUS = :statusActive
          AND TAS.STATUS = :statusActive
          AND TAS.PERCENT_COMPLETED < 100
          ORDER BY TTY.ORDER_TO_FRAME DESC
SQL;
        $params = array(
            'issueId' => $issueId,
            'typeLocked' => Entities\TaskTypeClassification::TYPE_LOCKED,
            'typeIssue' => Entities\TaskRelationship::RELATIONSHIP_ISSUE_TYPE,
            'statusActive' => Utilities::STATUS_ACTIVE,
            'usedInFraming' => Entities\Task::USED_IN_FRAMING,

        );
        $return = $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query, $params);

        return empty($return) ? null : $return[0];
    }
}
