<?php
namespace Teknisa\Model\Gestaopessoas\Repositories;

use HCM\Util\DateUtil;
use HCM\Util\Repositories;

class FpaItecalcfolha extends \Doctrine\ORM\EntityRepository {
    
    public function retornaItemPeriodo($nrorg, $nr_vinculo, $lista_nr_calculo, $lista_nr_evento){
      
        $lista_nr_evento = preg_split('/[\s,]+/', $lista_nr_evento);
        $lista_nr_calculo = preg_split('/[\s,]+/', $lista_nr_calculo);
        
        $params = array('nrorg'=>$nrorg, 'nrvinculom'=>$nr_vinculo, 'nreventom'=>$lista_nr_evento, 'nrcalculofolha'=>$lista_nr_calculo);
        $fpaIteCalcFolha = Repositories::FPA_ITECALCFOLHA;
        
$sql = <<<SQL
    SELECT i FROM $fpaIteCalcFolha i
    WHERE        
    i.nreventom IN (:nreventom)
    AND i.nrcalculofolha IN (:nrcalculofolha)
    AND i.nrvinculom = :nrvinculom
    AND i.nrorg = :nrorg
    ORDER BY i.nreventom, i.dtocorrencia DESC
SQL;
        
        $result = $this->getEntityManager()->createQuery($sql)->execute($params);
        
        return $result;
    }
    
    public function retorna3030SomadosNrFerias($nrorg, $nrferias, $nrvinculom, $nreventom){
        $param = array('NRORG'=> $nrorg, 'NRVINCULOM'=> $nrvinculom, 'NREVENTOM'=> $nreventom, 'NRFERIAS' => $nrferias);
        
$sql = <<<SQL
SELECT 
SUM(I.vreventofolha) AS vreventofolha
FROM fpa_itecalcfolha I
    JOIN fpa_calculofolha F ON F.nrorg = :NRORG
                            AND F.nrtipomoviment = 2
                            AND F.nrtpmodalidcal = 1
    JOIN fpa_ferias FE ON FE.nrorg = F.nrorg
                            AND FE.idcontrolferias = 'GOZADO_PARCIAL'
    JOIN fpa_gozoferias GF ON GF.nrorg = F.nrorg
                            AND GF.nrferias = FE.nrferias
WHERE I.nrorg = F.nrorg
AND I.nrvinculom = :NRVINCULOM
AND I.nrcalculofolha = F.nrcalculofolha
AND ( INSTR(','||:NREVENTOM||',', ','||I.NREVENTOM||',' ) > 0)
AND FE.nrferias = :NRFERIAS
AND I.nrgozoferias = GF.nrgozoferias
SQL;
            
        $result = \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql, $param);
        
        return empty($result) ? 0 : $result[0]['VREVENTOFOLHA'];
    }
    
    public function retornaValorPeriodoVinculoFiltros($nrorg, $nrtipomoviment, $dtini, $dtfim, $nrtpmodalidcal, $nrvinculom, $nreventom){
    
        $param = array('NRORG'=> $nrorg, 'NRVINCULOM'=> $nrvinculom, 'NREVENTOM'=> $nreventom, 'DTINI' => $dtini->format('d/m/Y'), 'DTFIM' => $dtfim->format('d/m/Y'), 'NRTIPOMOVIMENT' => $nrtipomoviment, 'NRTPMODALIDCAL' => $nrtpmodalidcal);
        
$sql = <<<SQL
SELECT 
SUM(I.vreventofolha) as VREVENTOFOLHA
FROM fpa_itecalcfolha I
    JOIN fpa_calculofolha F ON F.nrorg = :NRORG
    AND F.nrtipomoviment = :NRTIPOMOVIMENT
    AND F.dtocorrencia BETWEEN :DTINI AND :DTFIM
    AND F.nrtpmodalidcal = :NRTPMODALIDCAL
WHERE I.nrorg = F.nrorg
AND I.nrvinculom = :NRVINCULOM
AND I.nrcalculofolha = F.nrcalculofolha
AND I.nreventom = :NREVENTOM
SQL;
            
        $result = \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql, $param);
        
        return empty($result) ? 0 : $result[0]['VREVENTOFOLHA'];
    }
    
    public function retornaItemPeriodoVinculo($nrorg, $nr_vinculo, $nreventom, $periodoIni, $periodoFim){
        
        if(is_string($nreventom)){
            $nreventom = explode(',',$nreventom);
        }
        
        $params = array('nrorg'=> $nrorg, 'nrvinculom'=> $nr_vinculo, 'nreventom'=> $nreventom, 'periodoIni' => $periodoIni, 'periodoFim' => $periodoFim);
        $fpaIteCalcFolha = Repositories::FPA_ITECALCFOLHA;

$sql = <<<SQL
    SELECT i FROM $fpaIteCalcFolha i
    WHERE        
    i.nreventom IN(:nreventom)
    AND i.nrvinculom = :nrvinculom
    AND i.nrorg = :nrorg
    AND i.dtocorrencia >= :periodoIni AND i.dtocorrencia <= :periodoFim
    ORDER BY i.nreventom, i.dtocorrencia DESC
SQL;
        $result = $this->getEntityManager()->createQuery($sql)->execute($params);
        return $result;
    }
    
    public static function retornaItemPeriodoVinculoNormal($nrorg, $nr_vinculo, $nreventom, $periodoIni, $periodoFim, $dtmescompetenc)
    {
        $params = array('nrorg'=> $nrorg, 'nrvinculom'=> $nr_vinculo, 'nreventom'=> $nreventom, 'periodoIni' => $periodoIni, 'periodoFim' => $periodoFim, 'dtmescompetenc' => $dtmescompetenc);
        
        $query = "SELECT i.*, eh.IDUNIDREFEREVEN FROM FPA_ITECALCFOLHA i
                JOIN FPA_EVENTOH eh ON eh.nreventom = i.nreventom AND eh.dtmescompetenc = (SELECT MAX(eh2.dtmescompetenc) FROM FPA_EVENTOH eh2 WHERE eh2.nreventom = i.nreventom)
                WHERE        
                i.nreventom IN(:nreventom)
                AND i.nrvinculom = :nrvinculom
                AND i.nrorg = :nrorg
                AND i.dtocorrencia >= :periodoIni AND i.dtocorrencia <= :periodoFim
                ORDER BY i.nreventom, i.dtocorrencia DESC";

        return \Setforms2\Shared\Registry::getEntityManagerConnection()->fetchAll($query, $params);
    }
    
    
    
    public static function retornaItemGozoFeriasCalculado($nrorg, $nrcalculofolha, $nrvinculom, $nreventom, $nrgozoferias)
    {
        $query = "SELECT *
                FROM FPA_ITECALCFOLHA
                WHERE FPA_ITECALCFOLHA.NRORG = $nrorg
                AND FPA_ITECALCFOLHA.NRCALCULOFOLHA <> $nrcalculofolha
                AND FPA_ITECALCFOLHA.NRVINCULOM = $nrvinculom
                AND FPA_ITECALCFOLHA.NREVENTOM = $nreventom
                AND FPA_ITECALCFOLHA.NRGOZOFERIAS = $nrgozoferias";


        return \Setforms2\Shared\Registry::getEntityManagerConnection()->fetchAll($query);
    }
    
    public static function retornaTotalIncidCalculoPorEstrut($nrorg, $nrcalculofolha, $nrtipoincide, $nrestrutlegal, $nrestrutgeren, $dtmescompetenc)
    {

    
        $sql = <<<SQL
SELECT SUM(DECODE(IE.IDINCIDEVENTO,'+',IC.VREVENTOFOLHA,'-',IC.VREVENTOFOLHA*-1,0)) VRTOTAL
  FROM FPA_ITECALCFOLHA IC
  JOIN FPA_INCIDEVENTO IE ON (IE.NRORG = IC.NRORG OR IE.NRORG = 0)
                         AND IE.NREVENTOM = IC.NREVENTOM
                         AND IE.NRTIPOINCIDE = :P_NRTIPOINCIDE
  JOIN GPE_VINCULOH VH ON VH.NRORG = IC.NRORG
                      AND VH.NRVINCULOM = IC.NRVINCULOM
                      AND (VH.NRESTRUTLEGAL = :P_NRESTRUTLEGAL OR :P_NRESTRUTLEGAL = 'A')
                      AND (VH.NRESTRUTGEREN = :P_NRESTRUTGEREN OR :P_NRESTRUTGEREN = 'A')
                      AND VH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRORG = VH.NRORG
                                                  AND H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
 WHERE IC.NRORG = :P_NRORG
   AND IC.NRCALCULOFOLHA = :P_NRCALCULOFOLHA
SQL;
        $param = array(
            'P_NRORG'          => $nrorg,
            'P_NRCALCULOFOLHA' => $nrcalculofolha,
            'P_NRTIPOINCIDE'   => $nrtipoincide,
            'P_NRESTRUTLEGAL'  => $nrestrutlegal,
            'P_NRESTRUTGEREN'  => $nrestrutgeren,
            'P_DTMESCOMPETENC' => $dtmescompetenc);
        $result = \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql, $param);
        return empty($result) ? 0 : $result[0]['VRTOTAL'];
    }
    
    public function getValorEvento($nrcalculofolha,$nrvinculom, $nrorg, $nreventom, $dtpagamento){
      
        $params = array('nrorg'=>$nrorg, 'nrvinculom'=>$nrvinculom, 'nreventom'=>$nreventom, 'nrcalculofolha'=>$nrcalculofolha,
        'dtocorrencia' => $dtpagamento);
        
        $fpaIteCalcFolha = Repositories::FPA_ITECALCFOLHA;
        
$sql = <<<SQL
    SELECT i.vreventofolha FROM $fpaIteCalcFolha i
    WHERE        
    i.nreventom = :nreventom
    AND i.nrcalculofolha = :nrcalculofolha
    AND i.nrvinculom = :nrvinculom
    AND i.nrorg = :nrorg
    AND i.dtocorrencia >= :dtocorrencia
SQL;
        
        $result = $this->getEntityManager()->createQuery($sql)->execute($params);
        
        if($result){
            return $result[0]['vreventofolha'];
        }
        
        return null;
    }
    
    public function getItemCalculoEventoS1210($nrorg, $nrvinculom, $nreventom, $dtmescompetenc, $nrtipomoviment, $nrtpmodalidcal, $dtocorrencia){
        
        $params = array(
            'nrorg'=>$nrorg, 
            'nrvinculom'=>$nrvinculom, 
            'nreventom'=>$nreventom, 
            'dtmescompetenc'=>$dtmescompetenc,
            'nrtpmodalidcal'=>$nrtpmodalidcal, 
            'dtocorrencia_ini'=>DateUtil::getDataDeString($dtocorrencia, DateUtil::FORMATO_BRASILEIRO_DATAHORA)->format('01/m/Y'), 
            'dtocorrencia_fim'=>DateUtil::getDataDeString($dtocorrencia, DateUtil::FORMATO_BRASILEIRO_DATAHORA)->format('t/m/Y')
        );
        
        $fpaIteCalcFolha = Repositories::FPA_ITECALCFOLHA;
        $fpaCalculoFolha = Repositories::FPA_CALCULOFOLHA;
        
$sql = <<<SQL
    SELECT i
    FROM $fpaIteCalcFolha i
    JOIN $fpaCalculoFolha c WITH c.nrorg = i.nrorg
                             AND c.nrcalculofolha = i.nrcalculofolha
                             AND c.nrtpmodalidcal = :nrtpmodalidcal
                             AND c.dtocorrencia = :dtmescompetenc
    WHERE i.nrorg = :nrorg
    AND i.nrvinculom = :nrvinculom
    AND i.nreventom = :nreventom
    AND i.dtocorrencia between :dtocorrencia_ini and :dtocorrencia_fim
SQL;
        
        $result = $this->getEntityManager()->createQuery($sql)->execute($params);
        
        return current($result);
    }

    public function retornaItensDIRF($nrorg, $nrpessoa, $dtmescompetenc, $nrtpmodalidcal, $nrregimeapur, $dsordem, $dsinicio, $dsfim, $nmitem){
        
        $dsinicio = is_array($dsinicio) ? implode(',',$dsinicio) : $dsinicio;
        $dsfim = is_array($dsfim) ? implode(',',$dsfim) : $dsfim;
        
        $sql = <<<SQL
SELECT
    VM.NRPESSOA, VM.NRVINCULOM, VM.NMVINCULOM, VM.DTADMISSAOVINC, VM.DTRESCISAOVINC,
    REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(GIT.NMITEM, '-13S', ''), '-FERIAS', ''), '-ESTAG', ''), '-DIFF', ''), '-ADIANT-CX-ANT', ''), '-ADIANT-CX', '') NMITEM, /*Remove a idêntificação do 13º salário (este vai para o M13 e VALOR13)*/
    CASE
      WHEN ICF.NREVENTOM = 870 THEN '3562'
      ELSE LPAD(VE.CODRECEITA, 4, '0')
    END CDRECEITA,
    CASE
      /*2 0*/
        WHEN :nrregimeapur = 2 AND GIT.NMITEM = 'RTRT' AND RESC.DTOCORRENCIARESC IS NOT NULL THEN TO_CHAR(RESC.DTOCORRENCIARESC, 'yyyymm')
      WHEN INSTR(GIT.NMITEM, '-13S') > 0 THEN TO_CHAR(TO_DATE(:dtcompetencia), 'yyyy')||'13' /*Verifica se o item é do 13º salário (Ex.: se o id13Sal for '-13S' então 'RTRT-13S' é o M13 do 'RTRT')*/
      /*2 1*/ /*RegimeCaixa*/
        WHEN :nrregimeapur = 2 AND ICF.NREVENTOM = 1390 THEN TO_CHAR(CF.DTOCORRENCIA, 'yyyymm')
        WHEN :nrregimeapur = 2 AND CF.NRTIPOMOVIMENT = 1 AND (VM.DTRESCISAOVINC IS NULL OR TO_CHAR(CF.DTOCORRENCIA,'MMYYYY') <> TO_CHAR(VM.DTRESCISAOVINC,'MMYYYY')) THEN TO_CHAR(TO_DATE(:dtcompetencia), 'YYYY') || TO_CHAR(ADD_MONTHS(CF.DTOCORRENCIA, 1),'MM')
      WHEN GIT.NMITEM = 'RTPO-FERIAS' AND ICF.NREVENTOM IN (1140) THEN TO_CHAR(CF.DTOCORRENCIA, 'yyyymm')
      WHEN GIT.NMITEM = 'RTPO-FERIAS' AND ICF.NREVENTOM IN (1145) THEN TO_CHAR(CF.DTOCORRENCIA, 'yyyymm')
      /*1 8*/ /*RegimeCompetencia*/
        WHEN :nrregimeapur <> 2 AND GIT.NMITEM = 'RTPO' AND ICF.NREVENTOM IN (1140,1145) THEN TO_CHAR(ICF.DTOCORRENCIA, 'yyyymm')
      /*2 8*/ /*RegimeCaixa*/
        WHEN :nrregimeapur = 2 AND GIT.NMITEM = 'RTPO' AND ICF.NREVENTOM IN (1140,1145) THEN TO_CHAR(ADD_MONTHS(ICF.DTOCORRENCIA,1), 'yyyymm')
      /*2 2*/
        WHEN :nrregimeapur = 2 AND GIT.NMITEM = 'RTRT-ADIANT-CX' THEN TO_CHAR(ICF.DTOCORRENCIA, 'yyyymm')
        WHEN :nrregimeapur = 2 AND GIT.NMITEM = 'RTRT-ADIANT-CX-ANT' AND VM.DTRESCISAOVINC IS NOT NULL AND TO_CHAR(VM.DTRESCISAOVINC,'MMYYYY') = TO_CHAR(ICF.DTOCORRENCIA, 'MMYYYY') THEN TO_CHAR(ICF.DTOCORRENCIA, 'yyyymm')
        WHEN :nrregimeapur = 2 AND GIT.NMITEM = 'RTRT-ADIANT-CX-ANT' THEN TO_CHAR(ADD_MONTHS(ICF.DTOCORRENCIA, 1), 'yyyymm')
      ELSE TO_CHAR(CF.DTOCORRENCIA, 'yyyymm')
    END MES, /*são as colunas do pivô*/
    CASE
      WHEN GIT.NMITEM IN ('RTRT-13S','RTPO-13S') /*Zera itens do 13º na rescisao caso não possua data de rescisão*/
      AND VM.DTRESCISAOVINC IS NULL AND IEV.NREVENTOM IN (600,1180)
      THEN 0
      WHEN GIT.NMITEM IN ('RTRT-13S','RTPO-13S') /*Zera itens do 13º caso possua data de rescisão*/
      AND VM.DTRESCISAOVINC IS NOT NULL AND VM.DTRESCISAOVINC <= (TO_DATE(:dtcompetencia)+19) AND IEV.NREVENTOM IN (110,125,1150) /*Rescisoes ate o dia 20 nao tem 13o*/
      THEN 0
      WHEN GIT.NMITEM IN ('RTRT-ESTAG') /*Zera itens de estagiário quando o tipo de vínculo não for estagiário*/
      AND VM.NRTIPOVINCULOM <> 2 /*Estagiário*/
      THEN 0
      /*2 3*/
        WHEN :nrregimeapur = 2 AND GIT.NMITEM IN ('RTRT-ADIANT-CX') /*Subtrai o adiantamento do mês para entrar no mês anterior*/
        THEN DECODE(IEV.IDINCIDEVENTO, 'N', 0, DECODE(IEV.IDINCIDEVENTO, '-', -1 * ICF.VREVENTOFOLHA, ICF.VREVENTOFOLHA)) * (-1)
      ELSE DECODE(IEV.IDINCIDEVENTO, 'N', 0, DECODE(IEV.IDINCIDEVENTO, '-', -1 * ICF.VREVENTOFOLHA, ICF.VREVENTOFOLHA)) /*Considera o '+' e '-' no sum*/
    END VREVENTOFOLHA,
    ICF.NREVENTOM, ICF.DTOCORRENCIA, CF.DTOCORRENCIA AS DTOCORRENCIACALC, RESC.DTOCORRENCIARESC,
    GIT.NMITEM AS NMITEMORIG, GIT.VRPISOEVENTO, GIT.VRTETOEVENTO
  FROM FPA_CALCULOFOLHA CF
  JOIN FPA_ITECALCFOLHA ICF ON ICF.NRORG = CF.NRORG
                           AND ICF.NRCALCULOFOLHA = CF.NRCALCULOFOLHA
                           AND ICF.VREVENTOFOLHA > 0
  JOIN GPE_VINCULOM VM ON VM.NRORG = ICF.NRORG
                      AND VM.NRVINCULOM = ICF.NRVINCULOM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM (SELECT H1.DTMESCOMPETENC FROM GPE_VINCULOH H1
                                                WHERE H1.NRORG = VH.NRORG
                                                  AND H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.DTMESCOMPETENC <= CF.DTOCORRENCIA ORDER BY H1.DTMESCOMPETENC DESC) WHERE ROWNUM = 1)
  /*Código da receita do vínculo em questão (item 3.5)*/
  LEFT JOIN GPE_VINCULOEMPREG VE ON VE.NRVINCULOEMPREG = VM.NRVINCULOEMPREG
  /*Grupo de incidência da DIRF, Itens de Grupo e suas Incidências de eventos e os Itens de Cálculo*/
  JOIN (SELECT GIT.NRORG, GIT.NRITEM, GIT.NMITEM, GIT.NRGRPINCIDE, GIT.NRTIPOINCIDE, GIT.VRPISOEVENTO, GIT.VRTETOEVENTO
          FROM FPA_GRPITEM GIT
         WHERE (GIT.NRORG = :nrorg OR GIT.NRORG = 0)
           AND GIT.NRGRPINCIDE  = :nrgrupoinciden
        /*2 4*/ /*RegimeCaixa*/
        UNION ALL
        SELECT GIT.NRORG, GIT.NRITEM, 'RTRT-ADIANT-CX-ANT' AS NMITEM, GIT.NRGRPINCIDE, GIT.NRTIPOINCIDE, GIT.VRPISOEVENTO, GIT.VRTETOEVENTO
          FROM FPA_GRPITEM GIT
         WHERE (GIT.NRORG = :nrorg OR GIT.NRORG = 0)
           AND GIT.NRGRPINCIDE  = :nrgrupoinciden
           AND GIT.NMITEM = 'RTRT-ADIANT-CX'
           AND :nrregimeapur = 2) GIT ON ( GIT.NRORG = ICF.NRORG OR GIT.NRORG = 0 )
                                     AND GIT.NRGRPINCIDE  = :nrgrupoinciden
  JOIN FPA_INCIDEVENTO IEV ON ( IEV.NRORG = ICF.NRORG OR IEV.NRORG = 0 )
                          AND IEV.NRTIPOINCIDE = GIT.NRTIPOINCIDE
                          AND IEV.NREVENTOM = ICF.NREVENTOM
  LEFT JOIN (SELECT RESC.NRVINCULOM, RESC.DTOCORRENCIA AS DTOCORRENCIARESC, CFR.DTOCORRENCIA AS DTOCORRENCIACALC
               FROM FPA_CALCULOFOLHA CFR
               JOIN FPA_ITECALCFOLHA RESC ON RESC.NRORG = CFR.NRORG
                                         AND RESC.NRCALCULOFOLHA = CFR.NRCALCULOFOLHA
                                         AND RESC.NREVENTOM = 3050
              WHERE CFR.NRORG = :nrorg
                AND CFR.NRTIPOMOVIMENT = 1
                AND CFR.NRTPMODALIDCAL = 1
                AND CFR.NROCORRECAL = 1) RESC ON RESC.DTOCORRENCIACALC = TRUNC(VM.DTRESCISAOVINC,'MONTH')
                                             AND RESC.DTOCORRENCIACALC = CF.DTOCORRENCIA
                                             AND RESC.NRVINCULOM = VM.NRVINCULOM
 WHERE CF.NRORG = :nrorg
   AND CF.NRTPMODALIDCAL = :nrtpmodalidcal
   AND VM.NRPESSOA = :nrpessoa
   AND (VM.DTRESCISAOVINC IS NULL OR (VM.DTRESCISAOVINC >= TRUNC(TO_DATE(:dtcompetencia),'YEAR' )))
   AND ((:dsordem = 'ESTRUTURA'
            AND (VH.NRESTRUTLEGAL = :p_ds_inicio OR :p_ds_inicio IS NULL)
            AND (VH.NRESTRUTGEREN = :p_ds_fim OR :p_ds_fim IS NULL)) OR
        (:dsordem = 'LISTA_ESTRUTURA'
            AND ( INSTR(','||:p_ds_inicio||',', ','||VH.NRESTRUTLEGAL||',' ) > 0 OR :p_ds_inicio IS NULL )
            AND ( INSTR(','||:p_ds_fim||',', ','||VH.NRESTRUTGEREN||',' ) > 0 OR :p_ds_fim IS NULL )) OR
        (:dsordem = 'LISTA_NRESTRUTSIND'
            AND ( INSTR(','||:p_ds_inicio||',', ','||VH.NRESTRUTLEGAL||',' ) > 0 OR :p_ds_inicio IS NULL )
            AND ( INSTR(','||:p_ds_fim||',', ','||VH.NRESTRUTSIND||',' ) > 0 OR :p_ds_fim IS NULL )) OR
        (:dsordem <> 'ESTRUTURA' AND :dsordem <> 'LISTA_ESTRUTURA' AND :dsordem <> 'LISTA_NRESTRUTSIND')
       )
   AND (CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RIAP', 2)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RIAP-DIFF', 1)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTRT', 19)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTRT', 1)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTRT-FERIAS', 2)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTRT-ESTAG', 1)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTPO', 1)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTPO', 2)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTPO-FERIAS', 2)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTDP', 1)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTDP', 2)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTDP-13S', 4)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTDS', 1)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTDS', 2)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTDS-13S', 4)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTPA', 1)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RIIRP', 1)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTRT-13S', 8)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTRT-13S', 1)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTRT-13S', 4)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTRT-13S-INF', 8)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTRT-13S-INF', 1)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTRT-13S-INF', 4)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTPO-13S', 8)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTPO-13S', 1)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTPO-13S', 4)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTPA-13S', 8)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTPA-13S', 4)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTIRF-13S', 8)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTIRF-13S', 1)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTIRF-13S', 4)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTIRF', 1)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTIRF', 10)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTIRF', 2)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RIDAC', 1)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RIIRP', 1)
        OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RIO', 1)
        /*2 5*/ /*RegimeCaixa*/
            OR (:nrregimeapur = 2 AND CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTRT-ADIANT-CX', 10))
            OR (:nrregimeapur = 2 AND CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTRT-ADIANT-CX-ANT', 10)) )
        /*2 6*/
            AND ((:nrregimeapur = 2 AND CF.DTOCORRENCIA BETWEEN 
                                                DECODE(CF.NRTIPOMOVIMENT, 2, ADD_MONTHS(TRUNC(TO_DATE(:dtcompetencia),'YEAR'), -1),
                                                                            10, ADD_MONTHS(TRUNC(TO_DATE(:dtcompetencia),'YEAR'), -1),
                                                                            1, ADD_MONTHS(TRUNC(TO_DATE(:dtcompetencia),'YEAR'), -1),
                                                                            4, TO_DATE(:dtcompetencia),
                                                                            TRUNC(TO_DATE(:dtcompetencia),'YEAR'))
                                                AND 
                                                DECODE(CF.NRTIPOMOVIMENT, 1, DECODE(TO_CHAR(VM.DTRESCISAOVINC, 'MMYYYY'), TO_CHAR(CF.DTOCORRENCIA,'MMYYYY'), :dtcompetencia, ADD_MONTHS(:dtcompetencia, -1)), 
                                                                             :dtcompetencia)) OR
        /*1 6*/
                (:nrregimeapur <> 2 AND CF.DTOCORRENCIA BETWEEN 
                                                DECODE(CF.NRTIPOMOVIMENT, 2, ADD_MONTHS(TRUNC(TO_DATE(:dtcompetencia),'YEAR'),-1), 4, TO_DATE(:dtcompetencia), TRUNC(TO_DATE(:dtcompetencia ),'YEAR'))
                                                AND
                                                :dtcompetencia))
   AND ICF.NREVENTOM = DECODE(CF.NRTIPOMOVIMENT, 10, DECODE(ICF.NREVENTOM, 475, 475, DECODE(CF.DTOCORRENCIA, TO_DATE(:dtcompetencia), 1390, 0)), ICF.NREVENTOM)
   AND ICF.NREVENTOM = DECODE(CF.NRTIPOMOVIMENT,  1, DECODE(ICF.NREVENTOM, 205239, 0, 205543, 0, ICF.NREVENTOM), ICF.NREVENTOM) /*Apenas devem vir no movimento 4 (13º sal)*/
   AND GIT.NRITEM != 99 /*Todos os itens, exceto o 'informações complementares'*/
   AND GIT.NMITEM LIKE '%'||:nmitem||'%'
   AND EXISTS ( SELECT 1
                  FROM FPA_ITECALCFOLHA
                 WHERE NRORG = ICF.NRORG
                   AND NRCALCULOFOLHA = ICF.NRCALCULOFOLHA
                   AND NRVINCULOM = ICF.NRVINCULOM
                   AND NREVENTOM IN (SELECT NREVENTOM FROM FPA_INCIDEVENTO WHERE (NRORG = ICF.NRORG OR NRORG = 0) AND NRTIPOINCIDE = 109)
                   AND VREVENTOFOLHA > 0 )
SQL;

        $param = array(
            'nrorg' => $nrorg,
            'dtcompetencia' => $dtmescompetenc->format('d/m/Y'),
            'nrtpmodalidcal' => $nrtpmodalidcal,
            'nrgrupoinciden' => 10,
            'nrpessoa' => $nrpessoa,
            'nrregimeapur' => $nrregimeapur,
            'dsordem' => $dsordem,
            'p_ds_inicio' => $dsinicio,
            'p_ds_fim' => $dsfim,
            'nmitem' => $nmitem,
        );
        
        $result = \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql, $param);

        return $result;
    }
    
}