import { AlertServiceCustom } from '@/services/AlertServiceCustom';
import { DialogServiceCustom } from '@/services/DialogServiceCustom';
import { InstanceService } from '@/services/InstanceService';
import Messages from '@/services/Messages';
import { Utilities } from '@/services/Utilities';
import {
	DialogService,
	LoadingService,
	Modal,
	ModalService,
    TextInput,
} from '@zeedhi/common';
import { Http, Metadata } from '@zeedhi/core';

export class IssueCategoryController {
	public showIssueMainGrid = true;

	private addModal!: Modal;

	public issueCategoryLabel = '';

    public currentRow: any = {};

	public showBudgetItensCategoryForm() {
		return !this.showIssueMainGrid;
	}

	public clickBackCardForm() {
		this.showIssueMainGrid = true;
	}

	public async rowClickGridBudgetItensCategory(params: any) {
		this.issueCategoryLabel = params.row.LABEL;
        this.currentRow = params.row;
		this.showIssueMainGrid = false;
		setTimeout(async () => {
			const gridBudgetItensCategory = Metadata.getInstance<any>('gridBudgetItensCategory');
			gridBudgetItensCategory.datasource.filter.ID = params.row.ID;
			try {
				await gridBudgetItensCategory.datasource.get();
			} catch (e) {
				DialogServiceCustom.showErrorBackend(e);
			}
		}, 100);
	}

	public async openAddBudgetItensCategory() {
		if (this.addModal) {
			this.closeAddModal();
		}
		this.addModal = await ModalService.createFromJsonPost('', {
			jsonPath: 'issueConfiguration/modal/addBudgetItensCategory',
		});
		this.addModal.show();
	}

	public closeAddModal() {
		this.addModal.hide();
		this.addModal?.destroy();
	}

	public saveAddModal() {
		Utilities.genericAddModalTekGrid(
			'addBudgetItemForm',
			this.addModal,
			'/budgetItems/save',
			'gridIssueBudgetItens',
			true,
		);
	}

	public async beforeDeleteGridBudgetItensCategory(params: any) {
		params.event.preventDefault();
		const buttonYes = [
			{
				name: Utilities.getNameDynamic('yes-'),
				component: 'ZdButton',
				label: 'Sim',
				events: {
					click: () => {
						DialogService.hide();
						this.doDeleteGridBudgetItensCategory(params);
					},
				},
			},
		];
		DialogServiceCustom.confirmMessage(Messages.DELETE_DEFAULT, buttonYes);
	}

	public async doDeleteGridBudgetItensCategory(params: any) {
		LoadingService.show();
		try {
			await Http.post('/classificationBudgetItem/remove', {
				data: params.component.selectedRows,
			});
			params.component.selectedRows = [];
			params.component.datasource.get();
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
		}
		LoadingService.hide();
	}

	public async saveBudgetItensCategory() {
		const addBudgetItensCategoryForm = InstanceService.getInstanceZdForm('addBudgetItensCategoryForm');
		if (addBudgetItensCategoryForm.validate()) {
			const gridBudgetItensCategory = InstanceService.getInstanceTekGrid(
				'gridBudgetItensCategory',
			);
			LoadingService.show();
			try {
				await Http.post('/addBudgetItemInClassification/save', {
					ID: Metadata.getInstance<TextInput>('ID-BudgetItensCategoryForm').value,
					CLASSIFICATION: this.currentRow.ID,
				});
                this.closeAddModal();
				AlertServiceCustom.defaultSaveSuccessAlert();
				await gridBudgetItensCategory.datasource.get();
			} catch (e) {
				LoadingService.hide();
				DialogServiceCustom.showErrorBackend(e);
				throw e;
			}
			LoadingService.hide();
		} else {
			DialogServiceCustom.messageErrorValidate();
		}
	}

    public preencherAddBudgetItensCategoryForm({ component }: any) {
        Metadata.getInstance<TextInput>('ID-BudgetItensCategoryForm').value = component.selectValue.ID;
        Metadata.getInstance<TextInput>('CREATED_BY-BudgetItensCategoryForm').value = component.selectValue.CREATED_BY;
        Metadata.getInstance<TextInput>('CREATED_AT-BudgetItensCategoryForm').value = component.selectValue.CREATED_AT;
        Metadata.getInstance<TextInput>('VALUE_ITEM-BudgetItensCategoryForm').value = component.selectValue.VALUE_ITEM;
        Metadata.getInstance<TextInput>('UNIT_MEASURE-BudgetItensCategoryForm').value = component.selectValue.UNIT_MEASURE;
        Metadata.getInstance<TextInput>('STATUS-BudgetItensCategoryForm').value = component.selectValue.STATUS;
    }
}
