import { AlertServiceCustom } from '@/services/AlertServiceCustom';
import { DialogServiceCustom } from '@/services/DialogServiceCustom';
import { InstanceService } from '@/services/InstanceService';
import Messages from '@/services/Messages';
import { Utilities } from '@/services/Utilities';
import {
	DialogService,
	LoadingService,
	Modal,
	ModalService,
} from '@zeedhi/common';
import { Http } from '@zeedhi/core';

export class IssuePriorityController {
	public showIssueMainGrid = true;

	private addModal!: Modal;

	public priorityLabel = '';

	public showPriorityForm() {
		return !this.showIssueMainGrid;
	}

	public clickBackCardForm() {
		this.showIssueMainGrid = true;
	}

	public async rowClickGridPriority(params: any) {
		this.priorityLabel = params.row.LABEL;
		this.showIssueMainGrid = false;
		const priorityFormInstance = InstanceService.getInstanceZdForm('formPriority');
		priorityFormInstance.value = { ...params.row };
	}

	public async openAddPriorityModal() {
		if (this.addModal) {
			this.closeAddModal();
		}
		this.addModal = await ModalService.createFromJsonPost('', {
			jsonPath: 'issueConfiguration/modal/addPriorityModal',
		});
		this.addModal.show();
	}

	public closeAddModal() {
		this.addModal.hide();
		this.addModal?.destroy();
	}

	public saveAddModal() {
		Utilities.genericAddModalTekGrid(
			'addPriorityForm',
			this.addModal,
			'/issuePriority/save',
			'gridIssuePriority',
			true,
		);
	}

	public async beforeDeleteGridPriority(params: any) {
		params.event.preventDefault();
		const buttonYes = [
			{
				name: Utilities.getNameDynamic('yes-'),
				component: 'ZdButton',
				label: 'Sim',
				events: {
					click: () => {
						DialogService.hide();
						this.doDeleteGridPriority(params);
					},
				},
			},
		];
		DialogServiceCustom.confirmMessage(Messages.DELETE_DEFAULT, buttonYes);
	}

	public async doDeleteGridPriority(params: any) {
		LoadingService.show();
		try {
			await Http.post('/issuePriority/remove', {
				data: params.component.selectedRows,
			});
			params.component.selectedRows = [];
			params.component.datasource.get();
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
		}
		LoadingService.hide();
	}

	public async saveEditIssue() {
		const formPriority = InstanceService.getInstanceZdForm('formPriority');
		if (formPriority.validate()) {
			const gridCommentType = InstanceService.getInstanceTekGrid('gridIssuePriority');
			LoadingService.show();
			try {
				await Http.post('/issuePriority/save', {
					ID: formPriority.value.ID,
					LABEL: formPriority.value.LABEL,
					COLOR: formPriority.value.COLOR,
				});
				AlertServiceCustom.defaultSaveSuccessAlert();
				await gridCommentType.datasource.get();
			} catch (e) {
				LoadingService.hide();
				DialogServiceCustom.showErrorBackend(e);
				throw e;
			}
			LoadingService.hide();
		} else {
			DialogServiceCustom.messageErrorValidate();
		}
	}
}
