import Vue from 'vue';
import { ZdColorPicker } from '@zeedhi/zd-color-picker';
import {
	DialogService,
	Form,
	LoadingService,
	Modal,
	ModalService,
	Tabs,
} from '@zeedhi/common';
import { InstanceService } from '@/services/InstanceService';
import { Utilities } from '@/services/Utilities';
import { DialogServiceCustom } from '@/services/DialogServiceCustom';
import Messages from '@/services/Messages';
import { Http, Metadata } from '@zeedhi/core';
import { AlertServiceCustom } from '@/services/AlertServiceCustom';

Vue.component('ZdColorPicker', ZdColorPicker);

export class IssueStatusController {
	public showIssueMainGrid = true;

	private addModal!: Modal;

	private paramData: any;

	public issueStatus = '';

	public issueId: any;

	public showStatusForm() {
		return !this.showIssueMainGrid;
	}

	public clickBackCardForm() {
		this.showIssueMainGrid = true;
	}

	public rowClickGridStatus(params : any) {
		this.showIssueMainGrid = false;
		this.paramData = params.row;
		this.issueStatus = this.paramData.LABEL;
		this.issueId = this.paramData.ID;
        const tabs = Metadata.getInstance<Tabs>('tabIssueStatus');
		tabs.activeTab = 0;
        this.carregarTabsIssueStatus({ component: tabs });
    }

	public async carregarTabsIssueStatus({ component }: any) {
		const tabAtiva = component.activeTab;
		if (component.tabs[tabAtiva].name === 'tabStatusInfo') {
            this.mountTabInfo();
        }
		if (component.tabs[tabAtiva].name === 'tabStatusFraming') {
            this.createTabFraming();
			this.createGridTaskClassification();
			this.createGridProjectType();
        }
		if (component.tabs[tabAtiva].name === 'tabStatusComment') {
            this.createFormEntranceComment();
			this.createGridOutputComment();
        }
		if (component.tabs[tabAtiva].name === 'tabStatusAutoFraming') {
			this.createTabAutoFraming();
        }
	}

	public createTabAutoFraming() {
		let gridAutoFraming: any = null;
		setTimeout(async () => {
			gridAutoFraming = Metadata.getInstance<any>('gridAutoFraming');
			gridAutoFraming.datasource.filter.TASK_TYPE_ID = this.issueId;
			try {
				await gridAutoFraming.datasource.get();
			} catch (e) {
				DialogServiceCustom.showErrorBackend(e);
			}
		}, 100);
	}

	public mountTabInfo() {
		const infoFormInstance = InstanceService.getInstanceZdForm('formStatusInfo');
		infoFormInstance.value = {
			...this.paramData,
			CONTROL_NAME: this.paramData.CONTROL,
			ALLOW_CLOSE_ISSUE_NAME: this.paramData.ALLOW_CLOSE_ISSUE,
			ALLOW_PLAY_TASK_NAME: this.paramData.ALLOW_PLAY_TASK,
		};
	}

	public createTabFraming() {
		setTimeout(async () => {
			const framingFormInstance = InstanceService.getInstanceZdForm('formStatusFraming');
			framingFormInstance.value.ORDER_TO_FRAME = this.paramData.ORDER_TO_FRAME;
			framingFormInstance.value.ALLOW_PLAY_TASK_IN_FRAME_NAME = this.paramData.ALLOW_PLAY_TASK_IN_FRAME;
		}, 100);
	}

	public async openAddStatusModal() {
		if (this.addModal) {
			this.closeAddModal();
		}
		this.addModal = await ModalService.createFromJsonPost('', {
			jsonPath: 'issueConfiguration/modal/addStatusModal',
		});
		this.addModal.show();
	}

	public async openAddTaskClassificationModal() {
		if (this.addModal) {
			this.closeAddModal();
		}
		this.addModal = await ModalService.createFromJsonPost('', {
			jsonPath: 'issueConfiguration/modal/addTaskClassificationModal',
		});
		this.addModal.show();
	}

	public async openAddOutputCommentModal() {
		if (this.addModal) {
			this.closeAddModal();
		}
		this.addModal = await ModalService.createFromJsonPost('', {
			jsonPath: 'issueConfiguration/modal/outputCommentModal',
		});
		this.addModal.show();
	}

	public async openAddProjectTypeFormModal() {
		if (this.addModal) {
			this.closeAddModal();
		}
		this.addModal = await ModalService.createFromJsonPost('', {
			jsonPath: 'issueConfiguration/modal/addProjectTypeModal',
		});
		this.addModal.show();
	}

	public saveAddModal() {
		Utilities.genericAddModalTekGrid(
			'addStatusForm',
			this.addModal,
			'/taskType/save',
			'gridIssueStatus',
			false,
		);
	}

	public saveAddTaskClassificationModal() {
		Utilities.genericAddModalTekGrid(
			'addTaskClassificationForm',
			this.addModal,
			'/addTaskClassification/save',
			'gridTaskClassification',
			false,
		);
	}

	public saveAddProjectTypeModal() {
		Utilities.genericAddModalTekGrid(
			'addProjectTypeForm',
			this.addModal,
			'/addProjectType/save',
			'gridProjectType',
			false,
		);
	}

	public saveOutputCommentModal() {
		Utilities.genericAddModalTekGrid(
			'outputCommentForm',
			this.addModal,
			'/outputCommentIssueStatus/save',
			'gridOutputComment',
			false,
		);
	}

	public closeAddModal() {
		this.addModal.hide();
		this.addModal?.destroy();
	}

	public async beforeDeleteGridIssueStatus(params: any) {
		params.event.preventDefault();
		const buttonYes = [
			{
				name: Utilities.getNameDynamic('yes-'),
				component: 'ZdButton',
				label: 'Sim',
				events: {
					click: () => {
						DialogService.hide();
						this.doDeleteGridIssueStatus(params);
					},
				},
			},
		];
		DialogServiceCustom.confirmMessage(Messages.DELETE_DEFAULT, buttonYes);
	}

	public async doDeleteGridIssueStatus(params: any) {
		LoadingService.show();
		try {
			await Http.post('/taskType/remove', {
				data: params.component.selectedRows,
			});
			params.component.selectedRows = [];
			params.component.datasource.get();
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
		}
		LoadingService.hide();
	}

	public async beforeDeleteGridTaskClassification(params: any) {
		params.event.preventDefault();
		const buttonYes = [
			{
				name: Utilities.getNameDynamic('yes-'),
				component: 'ZdButton',
				label: 'Sim',
				events: {
					click: () => {
						DialogService.hide();
						this.doDeleteGridTaskClassification(params);
					},
				},
			},
		];
		DialogServiceCustom.confirmMessage(Messages.DELETE_DEFAULT, buttonYes);
	}

	public async doDeleteGridTaskClassification(params: any) {
		LoadingService.show();
		try {
			await Http.post('/issueFramingClassification/remove', {
				data: params.component.selectedRows,
			});
			params.component.selectedRows = [];
			params.component.datasource.get();
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
		}
		LoadingService.hide();
	}

	public async beforeDeleteGridProjectType(params: any) {
		params.event.preventDefault();
		const buttonYes = [
			{
				name: Utilities.getNameDynamic('yes-'),
				component: 'ZdButton',
				label: 'Sim',
				events: {
					click: () => {
						DialogService.hide();
						this.doDeleteGridProjectType(params);
					},
				},
			},
		];
		DialogServiceCustom.confirmMessage(Messages.DELETE_DEFAULT, buttonYes);
	}

	public async doDeleteGridProjectType(params: any) {
		LoadingService.show();
		try {
			await Http.post('/issueFramingProjectType/remove', {
				data: params.component.selectedRows,
			});
			params.component.selectedRows = [];
			params.component.datasource.get();
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
		}
		LoadingService.hide();
	}

	public async saveEditIssue() {
		const formStatusInfo = InstanceService.getInstanceZdForm('formStatusInfo');
		if (formStatusInfo.validate()) {
			const gridIssueStatus = InstanceService.getInstanceTekGrid('gridIssueStatus');
			LoadingService.show();
			try {
				await Http.post('/taskType/save', {
					ID: formStatusInfo.value.ID,
					LABEL: formStatusInfo.value.LABEL,
					CONTROL: formStatusInfo.value.CONTROL_NAME,
					ALLOW_CLOSE_ISSUE:
						formStatusInfo.value.ALLOW_CLOSE_ISSUE_NAME,
					ORDER_TO_VIEW: formStatusInfo.value.ORDER_TO_VIEW,
					TASK_CLASSIFICATION:
						formStatusInfo.value.TASK_CLASSIFICATION,
					ALLOW_PLAY_TASK: formStatusInfo.value.ALLOW_PLAY_TASK_NAME,
					ISSUE_CLASSIFICATION:
						formStatusInfo.value.ISSUE_CLASSIFICATION,
				});
				AlertServiceCustom.defaultSaveSuccessAlert();
				await gridIssueStatus.datasource.get();
			} catch (e) {
				LoadingService.hide();
				DialogServiceCustom.showErrorBackend(e);
				throw e;
			}
			LoadingService.hide();
		} else {
			DialogServiceCustom.messageErrorValidate();
		}
	}

	public async createGridTaskClassification() {
		let gridTaskClassification: any = null;
		setTimeout(async () => {
			gridTaskClassification = Metadata.getInstance<any>(
				'gridTaskClassification',
			);
			gridTaskClassification.datasource.filter.ISSUE = this.issueId;
			try {
				await gridTaskClassification.datasource.get();
			} catch (e) {
				DialogServiceCustom.showErrorBackend(e);
			}
		}, 100);
	}

	public async createGridProjectType() {
		let gridProjectType: any = null;
		setTimeout(async () => {
			gridProjectType = Metadata.getInstance<any>('gridProjectType');
			gridProjectType.datasource.filter.ISSUE = this.issueId;
			try {
				await gridProjectType.datasource.get();
			} catch (e) {
				DialogServiceCustom.showErrorBackend(e);
			}
		}, 100);
	}

	public async createGridOutputComment() {
		let gridOutputComment: any = null;
		setTimeout(async () => {
			gridOutputComment = Metadata.getInstance<any>('gridOutputComment');
			gridOutputComment.datasource.filter.ISSUE_STATUS = this.issueId;
			try {
				await gridOutputComment.datasource.get();
			} catch (e) {
				DialogServiceCustom.showErrorBackend(e);
			}
		}, 100);
	}

	public async createFormEntranceComment() {
		setTimeout(async () => {
			try {
				const response = await Http.post('/entranceCommentIssueStatus', {
					ISSUE_STATUS: this.issueId,
				});
				const formEntranceComment = Metadata.getInstance<Form>('formEntranceComment');
				if (response.data.data[0]) {
					let issueClassification = null;
					if (response.data.data[0].ISSUE_CLASSIFICATION) {
						issueClassification = response.data.data[0].ISSUE_CLASSIFICATION.split(',');
					}
					formEntranceComment.value = {
						COMMENT_TYPE_NAME: response.data.data[0].COMMENT_TYPE,
						DESCRIPTION: response.data.data[0].DESCRIPTION,
						TASK_TYPE: response.data.data[0].TASK_TYPE,
						ISSUE_CLASSIFICATION: issueClassification,
					};
				} else {
					formEntranceComment.value = {
						COMMENT_TYPE_NAME: null,
						DESCRIPTION: null,
						TASK_TYPE: this.issueId,
						ISSUE_CLASSIFICATION: null,
					};
				}
			} catch (e) {
				DialogServiceCustom.showErrorBackend(e);
			}
		}, 100);
	}

	public async rowClickOutputComment(params: any) {
		await this.openAddOutputCommentModal();
		const outputCommentFormInstance = InstanceService.getInstanceZdForm('outputCommentForm');
		outputCommentFormInstance.value = {
			...params.row,
			TASK_TYPE: this.issueId,
		};
	}

	public async addClickOutputComment() {
		await this.openAddOutputCommentModal();
		const outputCommentFormInstance = InstanceService.getInstanceZdForm('outputCommentForm');
		outputCommentFormInstance.value = {
			TASK_TYPE: this.issueId,
		};
	}

	public async addClickTaskClassification() {
		await this.openAddTaskClassificationModal();
		const addTaskClassificationFormInstance = InstanceService.getInstanceZdForm('addTaskClassificationForm');
		addTaskClassificationFormInstance.value = {
			TASK_TYPE: this.issueId,
		};
	}

	public async addClickProjectType() {
		await this.openAddProjectTypeFormModal();
		const addTaskClassificationFormInstance = InstanceService.getInstanceZdForm('addProjectTypeForm');
		addTaskClassificationFormInstance.value = {
			TASK_TYPE: this.issueId,
		};
	}

	public async beforeDeleteGridOutputComment(params: any) {
		params.event.preventDefault();
		const buttonYes = [
			{
				name: Utilities.getNameDynamic('yes-'),
				component: 'ZdButton',
				label: 'Sim',
				events: {
					click: () => {
						DialogService.hide();
						this.doDeleteGridOutputComment(params);
					},
				},
			},
		];
		DialogServiceCustom.confirmMessage(Messages.DELETE_DEFAULT, buttonYes);
	}

	public async doDeleteGridOutputComment(params: any) {
		LoadingService.show();
		try {
			await Http.post('/outputCommentIssueStatus/remove', {
				data: params.component.selectedRows,
			});
			params.component.selectedRows = [];
			params.component.datasource.get();
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
		}
		LoadingService.hide();
	}

	public async saveIssueOrderToFrame() {
		const formStatusFraming = InstanceService.getInstanceZdForm('formStatusFraming');
		if (formStatusFraming.validate()) {
			LoadingService.show();
			try {
				await Http.post('/saveIssueOrderToFrame', {
					ORDER_TO_FRAME: formStatusFraming.value.ORDER_TO_FRAME,
					ID: this.issueId,
					ALLOW_PLAY_TASK_IN_FRAME: formStatusFraming.value.ALLOW_PLAY_TASK_IN_FRAME_NAME,
				});
				AlertServiceCustom.defaultSaveSuccessAlert();
			} catch (e) {
				LoadingService.hide();
				DialogServiceCustom.showErrorBackend(e);
				throw e;
			}
			LoadingService.hide();
		} else {
			DialogServiceCustom.messageErrorValidate();
		}
	}

	public async saveEntranceCommentIssueStatus() {
		const formEntranceComment = InstanceService.getInstanceZdForm('formEntranceComment');
		if (formEntranceComment.validate()) {
			LoadingService.show();
			try {
				await Http.post('/saveEntranceCommentIssueStatus', {
					COMMENT_TYPE: formEntranceComment.value.COMMENT_TYPE_NAME,
					ISSUE_CLASSIFICATION: formEntranceComment.value.ISSUE_CLASSIFICATION,
					DESCRIPTION: formEntranceComment.value.DESCRIPTION,
					TASK_TYPE: formEntranceComment.value.TASK_TYPE,
				});
				AlertServiceCustom.defaultSaveSuccessAlert();
			} catch (e) {
				LoadingService.hide();
				DialogServiceCustom.showErrorBackend(e);
				throw e;
			}
			LoadingService.hide();
		} else {
			DialogServiceCustom.messageErrorValidate();
		}
	}

	public async openAddStatusAutoFramingModal(component: any) {
		if (this.addModal) {
			this.closeAddModal();
		}
		this.addModal = await ModalService.createFromJsonPost('', {
			jsonPath: 'issueConfiguration/modal/addStatusAutoFramingModal',
		});
		this.addModal.show();
		if (component.row) {
			setTimeout(async () => {
				const addStatusAutoFramingModalForm = InstanceService.getInstanceZdForm('addStatusAutoFramingModalForm');
				addStatusAutoFramingModalForm.value.ID = component.row.ID;
				addStatusAutoFramingModalForm.value.TIME = component.row.TIME;
				addStatusAutoFramingModalForm.value.TASK_STATUS_ACTUAL = component.row.TASK_STATUS_ACTUAL;
				addStatusAutoFramingModalForm.value.TASK_STATUS_DESTINY = component.row.TASK_STATUS_DESTINY;
				addStatusAutoFramingModalForm.value.DESCRIPTION = component.row.DESCRIPTION;
			}, 100);
		}
	}

	public async saveAddStatusAutoFramingModal() {
		const addStatusAutoFramingModalForm = InstanceService.getInstanceZdForm('addStatusAutoFramingModalForm');
		if (addStatusAutoFramingModalForm.validate()) {
			const gridAutoFraming = InstanceService.getInstanceTekGrid('gridAutoFraming');
			LoadingService.show();
			try {
				await Http.post('/saveAutomaticFraming', {
					ID: addStatusAutoFramingModalForm.value.ID,
					TIME: addStatusAutoFramingModalForm.value.TIME,
					TASK_STATUS_ACTUAL: addStatusAutoFramingModalForm.value.TASK_STATUS_ACTUAL,
					TASK_STATUS_DESTINY: addStatusAutoFramingModalForm.value.TASK_STATUS_DESTINY,
					TASK_TYPE_ID: this.paramData.ID,
					DESCRIPTION: addStatusAutoFramingModalForm.value.DESCRIPTION,
				});
				this.closeAddModal();
				AlertServiceCustom.defaultSaveSuccessAlert();
				await gridAutoFraming.datasource.get();
			} catch (e) {
				LoadingService.hide();
				DialogServiceCustom.showErrorBackend(e);
				throw e;
			}
			LoadingService.hide();
		} else {
			DialogServiceCustom.messageErrorValidate();
		}
	}

	public async removeAutomaticFraming(params: any) {
		LoadingService.show();
		try {
			await Http.post('/removeAutomaticFraming', {
				data: params.component.selectedRows,
			});
			params.component.selectedRows = [];
			params.component.datasource.get();
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
		}
		LoadingService.hide();
	}

	public async beforeDeleteAutomaticFraming(params: any) {
		params.event.preventDefault();
		const buttonYes = [
			{
				name: Utilities.getNameDynamic('yes-'),
				component: 'ZdButton',
				label: 'Sim',
				events: {
					click: () => {
						DialogService.hide();
						this.removeAutomaticFraming(params);
					},
				},
			},
		];
		DialogServiceCustom.confirmMessage(Messages.DELETE_DEFAULT, buttonYes);
	}
}
