import {
	Tree,
	ITreeClickEvent,
	ITreeNodeModel,
	DialogService,
	Form,
	IModal,
	Modal,
	ModalService,
	IMenuItem,
	Tabs,
} from '@zeedhi/common';
import {
	Http,
	I18n,
	IDictionary,
	IEventParam,
	Metadata,
} from '@zeedhi/core';
// @ts-ignore
import { saveAs } from 'file-saver';

export class LabelsController {
	private originalNodes: any;

	public modalFormTitle: string = '';

	public modalFormLabel: string = '';

	private editNode?: any;

	public modalTitle: string = '';

	private modal?: Modal;

	private modalOpen?: Modal;

	private modalImport?: Modal;

	public hasChanges: boolean = false;

	get hasNoChanges() {
		return !this.hasChanges;
	}

	public hasNoSelectedNodes: boolean = true;

	public editingFile: string = '';

	public beforeOpen: boolean = true;

	private version: number = 0;

	private modalDef: IModal = {
		name: 'labels-modal',
		persistent: true,
		children: [
			{
				name: 'labels-modal-header',
				component: 'ZdHeader',
				color: 'transparent',
				padless: true,
				elevation: 0,
				leftSlot: [
					{
						name: 'titleText',
						component: 'ZdText',
						text: '{{LabelsController.modalTitle}}',
						cssClass: 'zd-theme-font-title',
					},
				],
				rightSlot: [
					{
						name: 'closeModalButton',
						component: 'ZdModalCloseButton',
						small: true,
						modalName: 'labels-modal',
					},
				],
			},
			{
				name: 'labels-modal-form',
				component: 'ZdForm',
				cssClass: 'zd-mt-4',
				children: [
					{
						name: 'nodeTitle',
						component: 'ZdTextInput',
						label: 'NAME',
						autofocus: true,
						grid: {
							sm: 8,
						},
						value: '{{LabelsController.modalFormTitle}}',
						validations: {
							required: {},
						},
						keyMap: {
							enter: {
								event: this.modalSaveClick.bind(this),
								input: true,
								active: true,
							},
						},
					},
					{
						name: 'label',
						component: 'ZdTextInput',
						label: 'Label',
						grid: {
							sm: 4,
						},
						value: '{{LabelsController.modalFormLabel}}',
						keyMap: {
							enter: {
								event: this.modalSaveClick.bind(this),
								input: true,
								active: true,
							},
						},
					},
				],
			},
			{
				name: 'footer',
				component: 'ZdFooter',
				color: 'transparent',
				grid: {
					sm: '12',
				},
				padless: true,
				rightSlot: [
					{
						name: 'cancel-button',
						component: 'ZdButton',
						cssClass: 'zd-mr-2',
						outline: true,
						label: 'CANCEL',
						events: {
							click: this.modalCancelClick.bind(this),
						},
					},
					{
						name: 'save-button',
						component: 'ZdButton',
						label: 'SAVE',
						events: {
							click: this.modalSaveClick.bind(this),
						},
					},
				],
			},
		],
	};

	private modalOpenDef: IModal = {
		name: 'open-modal',
		children: [
			{
				name: 'open-modal-header',
				component: 'ZdHeader',
				color: 'transparent',
				padless: true,
				elevation: 0,
				leftSlot: [
					{
						name: 'titleText',
						component: 'ZdText',
						text: 'MESSAGE_OPEN',
						cssClass: 'zd-theme-font-title',
					},
				],
				rightSlot: [
					{
						name: 'closeButton',
						component: 'ZdButton',
						iconName: 'close',
						flat: true,
						icon: true,
						small: true,
						events: {
							click: this.modalOpenCancelClick.bind(this),
						},
					},
				],
			},
			{
				name: 'open-modal-form',
				component: 'ZdForm',
				children: [
					{
						name: 'file-open-modal',
						component: 'ZdFileInput',
						showDragArea: true,
						dragAreaHeight: '200px',
						showBorder: true,
						chips: true,
						showSize: true,
						maxFiles: 1,
						accept: '.json',
						grid: {
							sm: 12,
						},
						events: {
							change: this.modalOpenClick.bind(this),
						},
					}],
			},
		],
	};

	/** ---- Life cycle functions ----*/
	public treeCreated() {
		this.modal = ModalService.create(this.modalDef);
		this.modalOpen = ModalService.create(this.modalOpenDef);
		this.modalImport = ModalService.create(this.modalImportDef);
	}

	public treeBeforeDestroy() {
		this.modal?.destroy();
		this.modalOpen?.destroy();
		this.modalImport?.destroy();
	}
	/** ---- end life cycle functions ---- */

	/** ---- Label functions */
	public newLabel() {
		this.modalTitle = 'NEW_LABEL';
		this.modalFormTitle = '';
		this.modalFormLabel = '';
		this.editNode = undefined;
		this.modal?.show();
	}

	public deleteLabels(labelsTree: Tree) {
		labelsTree.removeSelected();
		this.save();
		this.hasNoSelectedNodes = true;
		DialogService.hide();
	}

	public openDialogDeleteLabels() {
		if (this.labelsTree.selectedNodes.length > 0) {
			DialogService.show({
				name: 'deleteDialog',
				type: 'warning',
				title: 'TITLE_DELETE',
				text: 'TEXT_DELETE',
				persistent: false,
				buttons: [
					{
						name: 'no',
						component: 'ZdButton',
						label: 'NO',
						outline: true,
						events: {
							click: () => {
								DialogService.hide();
							},
						},
					},
					{
						name: 'yes',
						component: 'ZdButton',
						label: 'YES',
						keyMap: {
							enter: {
								event: this.deleteLabels.bind(this, this.labelsTree),
							},
						},
						events: {
							click: this.deleteLabels.bind(this, this.labelsTree),
						},
					},
				],
			});
		}
	}
	/** ---- end label functions */

	/** ---- Modal functions ----*/
	private modalCancelClick() {
		this.modal?.hide();
	}

	private modalSaveClick() {
		const labelsForm = Metadata.getInstance<Form>('labels-modal-form');
		if (labelsForm.validate() && this.modal?.isVisible) {
			const newItem: any = {
				title: labelsForm.value.nodeTitle,
				data: {
					version: this.version + 1,
				},
			};
			if (labelsForm.value.label) {
				newItem.data.label = labelsForm.value.label;
			}

			if (!this.editNode) {
				this.labelsTree.insertNode(newItem);
				this.hasChanges = true;
			} else {
				this.labelsTree.updateNodeData(this.editNode, newItem);
				this.hasChanges = true;
			}

			this.modal?.hide();
		}
	}
	/** ---- end modal functions ----*/

	/** ---- Modal open functions ----*/
	public open() {
		this.beforeOpen = true;
		this.modalOpen?.show();
	}

	private modalOpenCancelClick() {
		this.modalOpen?.hide();
	}

	private modalOpenClick() {
		try {
			const openForm = Metadata.getInstance<Form>('open-modal-form');
			const readFile = openForm.value['file-open-modal'];
			const reader = new FileReader();

			reader.onload = (evt: any) => {
				const nodesString = evt.target.result.replaceAll('"isSelected":true', '"isSelected":false');
				this.labelsTree.nodes = JSON.parse(nodesString);
				this.version = this.getLastVersion();
				this.hasNoSelectedNodes = true;
				this.hasChanges = false;
			};

			if (openForm.validate()) {
				reader.readAsText(readFile);
				this.editingFile = readFile.name;
				this.modalOpenCancelClick();
			}
		} catch (error) {
			this.editingFile = '';
			DialogService.show({
				name: 'no-archive-dialog',
				type: 'error',
				title: 'ERRO_OPEN',
			});
		}
	}

	private getLastVersion() {
		let version = 0;
		this.labelsTree.nodes.forEach((node) => {
			if (node.data && node.data.version > version) {
				// eslint-disable-next-line prefer-destructuring
				version = node.data.version;
			}
			version = this.getLastChildrenVersion(node, version);
		});

		return version;
	}

	private getLastChildrenVersion(node: ITreeNodeModel<IDictionary<any>>, version: number) {
		let childrenVersion = version;
		if (node.children) {
			node.children.forEach((child) => {
				if (child.data && child.data.version > childrenVersion) {
					childrenVersion = child.data.version;
				}

				childrenVersion = this.getLastChildrenVersion(child, childrenVersion);
			});
		}

		return childrenVersion;
	}

	get afterOpen() {
		return !this.beforeOpen;
	}
	/** ---- end modal open functions ----*/

	private labelsTreeObject?: Tree;

	get labelsTree() {
		if (!this.labelsTreeObject) {
			this.labelsTreeObject = Metadata.getInstance<Tree>('tree');
		}
		return this.labelsTreeObject;
	}

	public onNodeSelect({ component }: IEventParam<Tree>) {
		this.hasNoSelectedNodes = component.selectedNodes.length === 0;
	}

	/** ---- events functions ---- */
	public onNodeDrop() {
		this.hasChanges = true;
	}

	public onNodeDblClick({ node }: ITreeClickEvent) {
		this.modalTitle = 'EDIT_LABEL';
		this.modalFormTitle = node.title;
		this.modalFormLabel = node.data?.label || '';
		this.editNode = node;
		this.modal?.show();
	}
	/** ---- end events functions ---- */

	/** ---- cancel button click ---- */
	public cancel() {
		if (this.originalNodes) {
			this.labelsTree.nodes = [...this.originalNodes];
		} else {
			this.labelsTree.nodes = [];
			this.editingFile = '';
		}
		this.hasChanges = false;
		this.labelsTree.clearSelection();
		DialogService.hide();
	}

	public openDialogCancelClick() {
		DialogService.show({
			name: 'cancelDialog',
			type: 'warning',
			title: 'CANCEL',
			text: 'TEXT_CANCEL',
			persistent: false,
			buttons: [
				{
					name: 'no',
					component: 'ZdButton',
					label: 'NO',
					outline: true,
					events: {
						click: () => {
							DialogService.hide();
						},
					},
				},
				{
					name: 'yes',
					component: 'ZdButton',
					label: 'YES',
					events: {
						click: this.cancel.bind(this),
					},
				},
			],
		});
	}
	/** ---- end cancel button click ---- */

	public getNodeText(node: ITreeNodeModel<any>) {
		return node.data.label;
	}

	public download() {
		let openForm;
		let nameArchive;
		const value = this.labelsTree.nodes;
		try {
			openForm = Metadata.getInstance<Form>('open-modal-form');
			nameArchive = openForm.value['file-open-modal'][0].name;
		} catch (error) {
			nameArchive = 'acl.json';
		}
		const blob = new Blob([JSON.stringify(value)], { type: 'text/plain;charset=utf-8' });
		saveAs(blob, nameArchive);
		this.hasChanges = false;
		this.originalNodes = [...value];
		this.version += 1;
	}

	public save() {
		const value = this.labelsTree.nodes;
		this.hasChanges = false;
		this.originalNodes = [...value];
	}

	private modalImportDef: IModal = {
		name: 'import-modal',
		children: [
			{
				name: 'import-modal-header',
				component: 'ZdHeader',
				color: 'transparent',
				padless: true,
				elevation: 0,
				leftSlot: [
					{
						name: 'titleText',
						component: 'ZdText',
						text: 'IMPORT',
						cssClass: 'zd-theme-font-title',
					},
				],
				rightSlot: [
					{
						name: 'closeModalButton',
						component: 'ZdModalCloseButton',
						small: true,
						modalName: 'import-modal',
					},
				],
			},
			{
				name: 'import-modal-tabs',
				component: 'ZdTabs',
				cssClass: 'zd-pt-4',
				tabs: [
					{
						name: 'import-file',
						tabTitle: 'IMPORT_FROM_FILE',
						children: [
							{
								name: 'import-modal-form-file',
								component: 'ZdForm',
								children: [
									{
										name: 'import-file',
										component: 'ZdFileInput',
										label: 'IMPORT_FROM_FILE',
										autofocus: true,
										grid: {
											sm: 12,
										},
										chips: true,
										showSize: true,
										showCounter: false,
										accept: '.json',
										validations: {
											required: {},
										},
									},
								],
							},
						],
					},
					{
						name: 'import-url',
						tabTitle: 'IMPORT_FROM_URL',
						children: [
							{
								name: 'import-modal-form-url',
								component: 'ZdForm',
								children: [
									{
										name: 'import-url',
										component: 'ZdTextInput',
										label: 'IMPORT_FROM_URL',
										grid: {
											sm: 12,
										},
										placeholder: 'http://',
										validations: {
											required: {},
										},
									},
								],
							},
						],
					},
					{
						name: 'import-text',
						tabTitle: 'IMPORT_FROM_TEXT',
						children: [
							{
								name: 'import-modal-form-text',
								component: 'ZdForm',
								children: [
									{
										name: 'json',
										label: 'IMPORT_FROM_TEXT',
										grid: {
											sm: 12,
										},
										component: 'ZdTextarea',
										validations: {
											required: {},
										},
									},
								],
							},
						],
					},
				],
			},
			{
				name: 'footer',
				component: 'ZdFooter',
				color: 'transparent',
				grid: {
					sm: '12',
				},
				padless: true,
				rightSlot: [
					{
						name: 'cancel-button',
						component: 'ZdButton',
						cssClass: 'zd-mr-2',
						outline: true,
						label: 'Cancel',
						events: {
							click: this.modalImportCancelClick.bind(this),
						},
					},
					{
						name: 'ok-button',
						component: 'ZdButton',
						label: 'OK',
						events: {
							click: this.modalImportOkClick.bind(this),
						},
					},
				],
			},
		],
	};

	public importMenu() {
		this.modalImport?.show();
	}

	private modalImportCancelClick() {
		this.modalImport?.hide();
	}

	private modalImportOkClick() {
		const importTabs = Metadata.getInstance<Tabs>('import-modal-tabs');
		let form: Form;
		switch (importTabs.activeTab) {
			case 0:
				form = Metadata.getInstance<Form>('import-modal-form-file');
				if (form.validate()) {
					this.loadFile();
				}
				break;
			case 1:
				form = Metadata.getInstance<Form>('import-modal-form-url');
				if (form.validate()) {
					this.loadURL();
				}
				break;
			case 2:
				form = Metadata.getInstance<Form>('import-modal-form-text');
				if (form.validate()) {
					this.import(form.value.json);
				}
				break;
			default:
				break;
		}
	}

	private import(text: string) {
		this.labelsTree.clearSelection();

		let menuNode: ITreeNodeModel<IDictionary<any>> | undefined;
		this.labelsTree.nodes.forEach((node) => {
			if (node.data?.label === 'MENU') {
				menuNode = node;
			}
		});

		if (!menuNode) {
			const index = this.labelsTree.nodes.push({
				title: 'menu',
				data: {
					label: 'MENU',
				},
			});
			menuNode = this.labelsTree.nodes[index - 1];
		}

		if (menuNode) {
			menuNode.isSelected = true;
		}

		this.labelsTree.importNodes(this.getNodesFromMenu(text));
		this.labelsTree.clearSelection();

		this.hasChanges = true;
		this.modalImport?.hide();
	}

	private getNodesFromMenu(json: string) {
		const importObject = JSON.parse(json);
		const items = importObject.dataset?.menu || importObject.menu || importObject;
		const nodes = this.getNodes(items);
		return nodes;
	}

	private getNodes(items: IMenuItem[]) {
		return items.map((item) => {
			const node: any = {
				title: item.label,
				data: {
					version: this.version + 1,
					label: item.label.toUpperCase().replaceAll(' ', '_'),
				},
			};

			if (item.items) {
				node.children = this.getNodes(item.items);
			} else if (item.menus?.length) {
				node.children = this.getNodes(item.menus);
			}

			return node;
		});
	}

	private loadFile() {
		try {
			const openForm = Metadata.getInstance<Form>('import-modal-form-file');
			const readFile = openForm.value['import-file'];
			if (readFile) {
				const reader = new FileReader();

				reader.onload = (evt: any) => {
					this.import(evt.target.result);
				};

				reader.readAsText(readFile);
			}
		} catch (error) {
			DialogService.show({
				name: 'no-archive-dialog',
				type: 'error',
				title: 'ERRO_LOAD_FILE',
			});
		}
	}

	static readonly ID_CLIENTE_LIBRARIES = 'cd7e85178153by60953518945f9hakt19a50a4';

	static readonly SECRET_CLIENTE_LIBRARIES = 'lib5449996c21f780fe30badd6521931a5sac9';

	private async loadURL() {
		try {
			const openForm = Metadata.getInstance<Form>('import-modal-form-url');
			const menuURL = openForm.value['import-url'];
			if (menuURL) {
				const route = `/${menuURL.split('/').pop()}`;
				const url = menuURL.substr(0, menuURL.indexOf(route));
				const token = await this.getLibToken(url);
				const secret = LabelsController.SECRET_CLIENTE_LIBRARIES;
				const filter = {
					requestType: 'FilterData',
					filter: [{
						name: 'CONCATENATE_GROUP_MENU',
						operator: '=',
						value: true,
					}],
				};

				const menu = await this.callWebService(token, route, filter, 'menu', secret, url);
				this.import(JSON.stringify(menu));
			}
		} catch (error) {
			DialogService.show({
				name: 'no-archive-dialog',
				type: 'error',
				title: `${I18n.translate('ERRO_LOAD_URL')}: '${error}'`,
			});
		}
	}

	private async getLibToken(url: string) {
		const secret = LabelsController.SECRET_CLIENTE_LIBRARIES;
		const param: IDictionary<any> = {
			CLIENT_ID: LabelsController.ID_CLIENTE_LIBRARIES,
			CLIENT_SECRET: secret,
		};
		const token = await this.callWebService('', '/lib_requestToken', param, 'token', secret, url);
		return token[0];
	}

	private async callWebService(
		token: string,
		tipoRequisicao: string,
		dados: IDictionary<any>,
		dataSetResultPos: string,
		chavePrivada: string,
		url: string,
		timeout: number = 18000,
	) {
		const headers = {
			'OAuth-Token': token,
			'OAuth-SecretId': chavePrivada,
			'Content-Type': 'application/json',
		};

		const config = {
			timeout,
			headers,
		};
		const result = await Http.post(url + tipoRequisicao, dados, config);

		if (result.data.dataset[dataSetResultPos]) {
			return result.data.dataset[dataSetResultPos];
		}

		if (Array.isArray(result.data)) {
			return result.data;
		}

		return null;
	}

	/**
	 * Teste
	 */
	public testeButton() {
		console.log(this.labelsTree.nodes);
		console.log(this.labelsTree.datasource);
		console.log(this.labelsTree.children);
	}
}
